<?php
    // This generator may be included directly by an include call, or invoked
    // through an HTTP request.
    
    if (@$wasIncluded == 0){
        require_once('../../../config.php');
        $u = optional_param('u', '*', PARAM_ALPHA);    // User id list if view is restricted
        $r = optional_param('r', '*', PARAM_ALPHA);    // Document URL or *
        $outputType = optional_param('outputType', 'GRAPHIC', PARAM_ALPHA);    // HTML or GRAPHIC
        $id = optional_param('id', 0, PARAM_INT);    // Course Module ID
    
        // get moodle context parameters
        if ($id) {
            if (! $cm = get_record('course_modules', 'id', $id)) {
                error('Course Module ID was incorrect');
            }
        
            if (! $coursetracker = get_record('coursetracking', 'id', $cm->instance)) {
                error("Coursetracker ID {$cm->instance} was incorrect");
            }
        
            if (! $course = get_record('course', 'id', $cm->course)) {
                error('Course is misconfigured');
            }
        } else {
            error('No valid parameters!!');
        }
    }

    $trace = 0;
    
    //graphical inits
    $CFG->systemfonts = "C:/WINNT/fonts/";
    $graphWidth = 350;
    $labelWidth = 350;
    $columnLabelHeight = 120;
    $lineHeight = 20;
    $columnWidth = 20;
    $ttffont = "Arial";
    
    define('VENI_FLAG', 0x01);
    define('VIDI_FLAG', 0x02);
    define('VICI_FLAG', 0x04);
    
    // initating tracing output for debugging
    if ($trace){
        $TRACE = fopen("outputgs.txt", "a");
        fwrite($TRACE, "GRAPH $r\n" );
    }
    
    // get documents filtering info

    if ($r == '*'){
       if ($trace) fwrite($TRACE, "all documents ($r)\n" );
       $documentIds = get_records_menu('course_modules', 'course', $course->id, 'instance');
       $documentList = '';
       if ($documentIds){
          $documentList = implode("','", array_keys($documentIds));
        }
    } else {
       $documentList = $r;
    }
    
    // computes some extra clauses

    $coursetrackerClause = ($coursetracker) ? " AND trackerid = {$coursetracker->id} " : '' ;
    $userRestrictionClause = '';
    if (isset($u) && $u != '*'){
        $u = str_replace(',', "','", $u);
        $userRestrictionClause = " AND u.id IN ('{$u}') ";
    }
    
    // Start compiling known markers for the current tracker

    $sql = "
        SELECT DISTINCT
            CONCAT(te.documentId,te.markerId) as uniqueId,
            te.documentId as documentId,
            te.markerId as markerId,
            cm.instance,
            cm.module
        FROM 
            {$CFG->prefix}coursetracking_event as te,
            {$CFG->prefix}course_modules as cm
        WHERE
            cm.id = te.documentid AND
            te.documentid IN ('{$documentList}')
            $coursetrackerClause
        ORDER BY
            te.documentid ASC,
            markerid
    ";
    $results = get_records_sql($sql);

    $markers = array();
    $distinctMarkers = array();
    $resources = array();
    if ($results){
        foreach(array_values($results) as $aMarker){
            // get the marker name knowing the course module
            if ($modulename = get_field('modules', 'name', 'id', $aMarker->module)){
                $aMarker->name = get_field($modulename, 'name', 'id', $aMarker->instance);
            } else {
                $aMarker->name = 'unkown';
            }

            // get marker keys in order
            $markers[$aMarker->documentId][$aMarker->markerId] = 1;
            $distinctMarkers[$aMarker->documentId . '_' . $aMarker->markerId] = 1;
            $resources[$aMarker->documentId] = $aMarker->name;
        }
    }
    
    
    // get marker datas and compile some browsable structure
    // information is fetched by user
    $sql = "
        SELECT
            te.*,
            r.name as name,
            cm.id as cmid,
            u.firstname,
            u.lastname,
            u.username,
            u.email,
            te.userid = {$USER->id} as isme
        FROM 
            {$CFG->prefix}coursetracking_event as te,
            {$CFG->prefix}course_modules as cm,
            {$CFG->prefix}resource as r,
            {$CFG->prefix}user as u
        WHERE
            cm.id = te.documentid AND
            cm.instance = r.id AND
            te.documentid IN ('{$documentList}') AND
            te.userid = u.id
            $coursetrackerClause
            $userRestrictionClause
        ORDER BY
            te.documentid ASC,
            markerid,
            u.lastname
    ";
    $results = get_records_sql($sql);

    $students = array();
    $totalViewTimeMax = 0;
    $perMarkerViewTimeMax = array();
    if ($results){
        foreach(array_values($results) as $aMarker){
            $userKey = $aMarker->lastname . $aMarker->firstname;
            if (!isset($students[$userKey]->status))
                $students[$userKey]->status = array();
            if (!isset($students[$userKey]->vidiTime))
                $students[$userKey]->vidiTime = array();
            if ($aMarker->event == 'veni'){
                $students[$userKey]->status[$aMarker->markerid] = @$students[$userKey]->status[$aMarker->markerid] | VENI_FLAG;
            }
            if ($aMarker->event == 'vidi'){
                $students[$userKey]->status[$aMarker->markerid] = @$students[$userKey]->status[$aMarker->markerid] | VIDI_FLAG;
            }
            if ($aMarker->event == 'vici'){
                $students[$userKey]->status[$aMarker->markerid] = @$students[$userKey]->status[$aMarker->markerid] | VICI_FLAG;
            }
            if ($students[$userKey]->status[$aMarker->markerid] > 1 && $aMarker->event == 'vidi') {
                $students[$userKey]->vidiTime[$aMarker->markerid] = $aMarker->totaltime;
                // absolute max for vidi time over population
                if ($aMarker->totaltime > $totalViewTimeMax) $totalViewTimeMax = $aMarker->totaltime;
                // relative max for vidi time on this marker
                if ($aMarker->totaltime > @$perMarkerViewTimeMax[$aMarker->documentid.'_'.$aMarker->markerid]) 
                    $perMarkerViewTimeMax[$aMarker->documentid.'_'.$aMarker->markerid] = $aMarker->totaltime;
            }
            if (!isset($students[$userKey]->user)){
                $aUser->firstname = $aMarker->firstname;
                $aUser->lastname = $aMarker->lastname;
                $aUser->username = $aMarker->username;
                $aUser->id = $aMarker->userid;
                $students[$userKey]->user = $aUser;
            }
        }
    }
    asort($students);
    
    // check some extreme situations

    if (count($students) == 0){
        $textmessage = get_string('notrackingresults', 'coursetracking');
    }
    
    /*
    *
    * This is the HTML Table non graphic output generation
    *
    */
    if ($outputType == 'HTML'){
        print_heading(get_string('studentmaptitle', 'coursetracking'), 2);
        echo "<link type=\"text/css\" rel=\"stylesheet\" href=\"{$CFG->wwwroot}/mod/coursetracking/coursetracking.css\">";

        if (isset($textmessage)){
            echo "<span>{$textmessage}</span>";
            return;
        }

        unset($table);
        $table->head = array();
    	$table->head[0] = get_string('markerid', 'coursetracking');
    	$table->align[0] = 'left';
    	$table->size[0] = '40%';
    	$table->wrap[0] = '';
    	$table->data = array();
    	$i = 0;
        foreach(array_keys($markers) as $aDocumentId){
            $table->data[$i][0] = "<img src=\"{$CFG->wwwroot}/mod/resource/icon.gif\"> <a class=\"documentLink\" href=\"{$CFG->wwwroot}/mod/resource/view.php?id={$aDocumentId}\">" . $resources[$aDocumentId] . "</a>";
            $i++;
            foreach(array_keys($markers[$aDocumentId]) as $aMarkerId){
            	$table->data[$i][0] = "<img src=\"{$CFG->wwwroot}/mod/coursetracking/icon.gif\"> <a class=\"trackerLink\" href=\"{$CFG->wwwroot}/mod/resource/view.php?id={$aDocumentId}#lnk_{$aMarkerId}\">" . $aMarkerId . "</a>";
            	$i++;
            }
        }
        $i = 1;
    	foreach(array_keys($students) as $aStudentKey){
    	    // TODO : match Moodle config on user name presentation
    	    ($students[$aStudentKey]->user->id == $USER->id) ? $class = "isme" : $class = '';
    	    $studentname = "<span class=\"$class\">{$students[$aStudentId]->user->firstname} {$students[$aStudentId]->user->lastname}</span>";
    	    $table->head[$i] = "{$studentname}";
    	    $table->align[$i] = 'center';
    	    $table->size[$i] = "";
    	    $table->wrap[$i] = '';
            $j = 0;
            foreach(array_keys($markers) as $aDocumentId){
                $table->data[$j][$i] = '';
                $j++; 
                foreach(array_keys($markers[$aDocumentId]) as $aMarkerId){
                	$table->data[$j][$i] = '<img src="none.gif" border="0" />'; 
                    if ($students[$aStudentKey]->status[$aMarkerId] & VENI_FLAG){
                	    $table->data[$j][$i] = '<img src="veni.gif" border="0" />'; 
                	}
                    if ($students[$aStudentKey]->status[$aMarkerId] & VIDI_FLAG){
                	    $table->data[$j][$i] = '<img src="vidi.gif" border="0" />'; 
                	    if ($students[$aStudentKey]->vidiTime[$aMarkerId] != 0){
                	        $table->data[$j][$i] .= ' (' . $students[$aStudentKey]->vidiTime[$aMarkerId]. '/' . $perMarkerViewTimeMax[$aMarkerId] . ' sec.)'; 
                	    }
                	}
                    if ($students[$aStudentKey]->status[$aMarkerId] & VICI_FLAG){
                	    $table->data[$j][$i] = '<img src="vici.gif" border="0" />'; 
                	    if ($students[$aStudentKey]->vidiTime[$aMarkerId] != 0){
                	        $table->data[$j][$i] .= ' (' . $students[$aStudentKey]->vidiTime[$aMarkerId]. '/' . $perMarkerViewTimeMax[$aMarkerId] . ' sec.)'; 
                	    }
                	}
                	$j++;
                }
            }
            $i++;
        }
    	$table->class = '';
    	$table->fontsize = 2;
    	$table->tablealign = 'left';
    	$table->width = '80%';
    	$table->cellpadding = 2;
    	$table->cellspacing = 0;
        echo make_table($table);
        unset($table);
        unset($students);
        unset($markers);
        unset($resources);
    
    } else {
    /*
    *
    * This is the GD generation alternative
    *
    */
        header("Content-type: image/png");
      
        // Searching for font files

        $fontFile = $CFG->dirroot . "/mod/coursetracking/fonts/arial.ttf";
        if (!file_exists($fontFile))
            $fontFile = $CFG->dirroot . "/fonts/{$ttffont}.ttf";
        // if fonts where not given by project, try with system fonts.
        if (!file_exists($fontFile))
            $fontFile = @$CFG->systemfonts . "/{$ttffont}.ttf";
    
        // output special situations messages 

        if (isset($textmessage)){
            $bounds = imagettfbbox(14, 0, $fontFile, $textmessage);
            $imageWidth = $bounds[2] - $bounds[0] + 2;
            $imageHeight = ($bounds[1] - $bounds[7]) * 1.4;
        } else {
            $imageWidth = $labelWidth + count($students) * $columnWidth + 10;
            $imageHeight = $columnLabelHeight  + $lineHeight * (count(array_keys($distinctMarkers)) + count($resources)) + 10 ;
        }
    
        $im = imagecreatetruecolor($imageWidth, $imageHeight);
        imageantialias($im, TRUE);
    
        // Assigning colors

        $colors['black'] = imagecolorallocate($im, 0, 0, 0);
        $colors['quiteblack'] = imagecolorallocate($im, 10, 10, 10);
        $colors['green'] = imagecolorallocate($im, 0, 240, 0);
        $colors['yellow'] = imagecolorallocate($im, 230, 230, 0);
        $colors['red'] = imagecolorallocate($im, 200, 0, 0);
        $colors['white'] = imagecolorallocate($im, 240, 240, 240);
        $colors['lightgray'] = imagecolorallocate($im, 200, 200, 200);
        $colors['gray'] = imagecolorallocate($im, 150, 150, 150);
        $colors['darkgray'] = imagecolorallocate($im, 100, 100, 100);
        $colors['blue'] = imagecolorallocate($im, 0, 0, 180);
        $colors['isme'] = imagecolorallocate($im, 160, 160, 0);
        
        imagefilledrectangle($im, 0, 0, $imageWidth, $imageHeight, $colors['white']);
    
        if (!isset($textmessage)){
            /// Creating graph
    
            // Creating title

            imagettftext($im, 18, 0, 10, 26, $colors['black'], $fontFile, $r);
        
            // draw background

            imagefilledrectangle($im, $labelWidth + 10, $columnLabelHeight, $imageWidth, $imageHeight, $colors['gray']);
    
            // draw labels for documents and markers
            $i = 1;
            foreach(array_keys($markers) as $aDocumentId){
                $y = $columnLabelHeight + ($i * $lineHeight);
                imagettftext($im, 12, 0, 5, $y, $colors['black'], $fontFile, $resources[$aDocumentId]);
                $i++;
                foreach(array_keys($markers[$aDocumentId]) as $aMarkerId){
                    $y = $columnLabelHeight + ($i * $lineHeight);
                    imagettftext($im, 10, 0, 15, $y, $colors['blue'], $fontFile, $aMarkerId);
                    imageline($im, $labelWidth + 10, $y + 5, $imageWidth, $y + 5, $colors['lightgray']);
                    imageline($im, $labelWidth + 10, $y + 6, $imageWidth, $y + 6, $colors['darkgray']);
                	$i++;
                }
            }
    
            // draw marker ellipses
            $i = 1;
        	foreach(array_keys($students) as $aStudentKey){
        	    // TODO : match Moodle config on user name presentation
        	    $x = $labelWidth + $i * $columnWidth;
        	    ($students[$aStudentKey]->user->id == $USER->id) ? $color = "isme" : $color = 'black';
                imagettftext($im, 10, 90, $x + 3, 118, $colors[$color], $fontFile, $aStudentKey);
                $j = 0;
                foreach(array_keys($markers) as $aDocumentId){
                    $j++; 
                    foreach(array_keys($markers[$aDocumentId]) as $aMarkerId){
                    	$color = 'black'; 
                        if (@$students[$aStudentKey]->status[$aMarkerId] & VENI_FLAG){
                    	    $color = 'red'; 
                    	}
                        if (@$students[$aStudentKey]->status[$aMarkerId] & VIDI_FLAG){
                    	    $color = 'yellow'; 
    //                	    if ($students[$aStudentKey]->vidiTime[$aMarkerId] != 0)
    //                	        $table->data[$j][$i] .= ' (' . $students[$aStudentKey]->vidiTime[$aMarkerId]. ' sec.)'; 
                    	}
                        if (@$students[$aStudentKey]->status[$aMarkerId] & VICI_FLAG){
                    	    $color = 'green'; 
    //                	    if ($students[$aStudentKey]->vidiTime[$aMarkerId] != 0)
    //                	        $table->data[$j][$i] .= ' (' . $students[$aStudentKey]->vidiTime[$aMarkerId]. ' sec.)'; 
                    	}
                    	$y = $columnLabelHeight + $lineHeight * $j;
                        imagefilledellipse($im, $x, $y + 15, 10, 10, $colors[$color]);
                    	$j++;
                    }
                }
                $i++;
            }
        } else {
            imagettftext($im, 12, 0, 5, $imageHeight - 4, $colors['black'], $fontFile, $textmessage);
        }
    
      // delivering image
      imagepng($im);
      imagedestroy($im);
    }
?>