<?php
    // This generator may be included directly by an include call, or invoked
    // through an HTTP request.
    global $prevx;
    global $prevy;
    
    /*
    This generator generates graphic curves of the dynamic progress of students over the content.
    */
    if (@$wasIncluded == 0){
        require_once('../../../config.php');
        $u = optional_param('u', '*', PARAM_CLEAN);    // User id list if view is restricted
        $r = optional_param('r', '*', PARAM_CLEAN);    // Document URL or *
        $outputType = optional_param('outputType', 'GRAPHIC', PARAM_ALPHA);    // HTML or GRAPHIC
        $id = optional_param('id', 0, PARAM_INT);    // Course Module ID
        $showVeni = optional_param('veni', 1, PARAM_INT);    // show Veni Data
        $showVidi = optional_param('vidi', 1, PARAM_INT);    // show Vidi Data
        $showVici = optional_param('vici', 1, PARAM_INT);    // show Vici Data
        $columnWidth = $SESSION->zoom;    // zoom factor
        $trackOffset = $SESSION->offset;    // track display offset
    
        // get moodle context parameters
        if ($id) {
            if (! $cm = get_record('course_modules', 'id', $id)) {
                error('Course Module ID was incorrect');
            }
        
            if (! $coursetracker = get_record('coursetracking', 'id', $cm->instance)) {
                error("Coursetracker ID {$cm->instance} was incorrect");
            }
        
            if (! $course = get_record('course', 'id', $cm->course)) {
                error('Course is misconfigured');
            }
        } else {
            error('No valid parameters!!');
        }
    }
    
    global $TRACE;
    global $trace;
    $trace = 1;
    
    // define some functions
    
    function drawBox($im, $x, $y, $color, $colors, $joinBoxes){
        global $prevx;
        global $prevy;
        global $TRACE;
        global $trace;
    
        if ($trace) fwrite($TRACE, "plotting $x, $y $color\n");
        
        if ($joinBoxes){
            imageline($im, $prevx + 2, $prevy + 2, $x + 2, $y + 2, $colors[$color]);
        }
        imagefilledrectangle($im, $x, $y, $x + 5, $y + 5, $colors['black']);
        imagefilledrectangle($im, $x + 1, $y + 1, $x + 4, $y + 4, $colors[$color]);
        $prevx = $x;
        $prevy = $y;
    }
    
    // Graphical parameters
    
    $CFG->systemfonts = "C:/WINNT/fonts/";
    $graphWidth = 350;
    $labelWidth = 250;
    $columnLabelHeight = 100;
    $lineHeight = 10;
    
    // $columnWidth = 200;
    $maxImageWidth = 800;
    $ttffont = "Arial";
    $vidiTimeBarLength = 30;
    
    define('VENI_FLAG', 0x01);
    define('VIDI_FLAG', 0x02);
    define('VICI_FLAG', 0x04);
    
    // initiating tracing output for debugging
    
    if ($trace) {
        $TRACE = fopen("outputgs.txt", "a");
        fwrite($TRACE, "GRAPH $r\n" );
    }
    
    // get documents filtering info
    
    if ($r == '*'){
       if ($trace) fwrite($TRACE, "all documents ($r)\n" );
       $documentIds = get_records_menu('course_modules', 'course', $course->id, 'instance');
       $documentList = '';
       if ($documentIds)
          $documentList = implode("','", array_keys($documentIds));
    } else {
       $documentList = str_replace(",", "','", $r);
    }
    
    // Getting calendar related information

    $trackstartdate = ($coursetracker->trackstartenable) ? $coursetracker->trackstart : $coursetracker->capturestart ; 
    $sql = "
       SELECT 
        UNIX_TIMESTAMP(NOW()) - {$trackstartdate} as secsFromOrigin,
        DAYOFWEEK(FROM_UNIXTIME({$trackstartdate})) as dayOfWeek,
        MONTH(FROM_UNIXTIME({$trackstartdate})) as month,
        YEAR(FROM_UNIXTIME({$trackstartdate})) as year
    ";
    $calendarInfo = get_record_sql($sql);
    /*
    dayOfWeek = 1 for Sunday, 2 for Monday ...
    */
    
    // computes some extra clauses

    $coursetrackerClause = ($coursetracker) ? " AND trackerid = {$coursetracker->id} " : '' ;
    $userRestrictionClause = '';
    if (isset($u) && $u != '*'){
        $u = str_replace(',', "','", $u);
        $userRestrictionClause = " AND u.id IN ('{$u}') ";
    }
    
    // Start compiling known markers for the current tracker

    $sql = "
        SELECT DISTINCT
            CONCAT(te.documentid,te.markerid) as uniqueId,
            te.documentid as documentid,
            te.markerid as markerid,
            cm.instance,
            cm.module
        FROM 
            {$CFG->prefix}coursetracking_event as te,
            {$CFG->prefix}course_modules as cm
        WHERE
            cm.id = te.documentid AND
            te.documentid IN ('{$documentList}')
            $coursetrackerClause
        ORDER BY
            te.documentid ASC,
            markerid
    ";
    $results = get_records_sql($sql);

    $markers = array();
    $distinctMarkers = array();
    $resources = array();
    $i = 0;
    if ($results){
        foreach(array_values($results) as $aMarker){
            // get the marker name knowing the course module
            if ($modulename = get_field('modules', 'name', 'id', $aMarker->module)){
                $aMarker->name = get_field($modulename, 'name', 'id', $aMarker->instance);
            } else {
                $aMarker->name = 'unkown';
            }

            // get marker keys in order and store a reverse index
            $resources[$aMarker->documentid] = $aMarker->name;
            $markers[$aMarker->documentid][$aMarker->markerid] = $i + count(array_keys($resources));
            $distinctMarkers[$aMarker->documentid . '_' . $aMarker->markerid] = $i + count(array_keys($resources));
            $i++;
        }
    }
    
    // get marker datas and compile some browsable structure
    // information is fetched by user
    $sql = "
        SELECT
            te.*,
            UNIX_TIMESTAMP(firstoccurred) - {$trackstartdate} as secsFromOrigin,
            r.name as name,
            cm.id as cmid,
            u.firstname,
            u.lastname,
            u.username,
            u.email,
            te.userid = {$USER->id} as isme
        FROM 
            {$CFG->prefix}coursetracking_event as te,
            {$CFG->prefix}course_modules as cm,
            {$CFG->prefix}resource as r,
            {$CFG->prefix}user as u
        WHERE
            cm.id = te.documentid AND
            cm.instance = r.id AND
            te.documentid IN ('{$documentList}') AND
            te.userid = u.id
            $coursetrackerClause
            $userRestrictionClause
        ORDER BY
            te.firstoccurred,
            te.documentid ASC,
            markerid,
            u.lastname
    ";
    $results = get_records_sql($sql);

    $students = array();
    $totalViewTimeMax = 0;
    $perMarkerViewTimeMax = array();
    if ($results){
        foreach(array_values($results) as $aMarker){
            if (!isset($students[$aMarker->userid]->venis))
                $students[$aMarker->userid]->venis = array();
            if (!isset($students[$aMarker->userid]->vidis))
                $students[$aMarker->userid]->vidis = array();
            if (!isset($students[$aMarker->userid]->vicis))
                $students[$aMarker->userid]->vicis = array();
    
            if (!isset($students[$aMarker->userid]->vidiTime))
                $students[$aMarker->userid]->vidiTime = array();
            if ($aMarker->event == 'veni'){
                $students[$aMarker->userid]->venis[$aMarker->secsFromOrigin][] = $aMarker->documentid . '_' . $aMarker->markerid;
            }
            if ($aMarker->event == 'vidi'){
                $students[$aMarker->userid]->vidis[$aMarker->secsFromOrigin][] = $aMarker->documentid . '_' . $aMarker->markerid;
            }
            if ($aMarker->event == 'vici'){
                $students[$aMarker->userid]->vicis[$aMarker->secsFromOrigin][] = $aMarker->documentid . '_' . $aMarker->markerid;
            }
            if ($aMarker->event == 'vidi') {
                $students[$aMarker->userid]->vidiTime[$aMarker->documentid . '_' . $aMarker->markerid] = $aMarker->totaltime;
                // absolute max for vidi time over population
                if ($aMarker->totaltime > $totalViewTimeMax) $totalViewTimeMax = $aMarker->totaltime;
                // relative max for vidi time on this marker
                if ($aMarker->totaltime > @$perMarkerViewTimeMax[$aMarker->documentid . '_' . $aMarker->markerid]) 
                    $perMarkerViewTimeMax[$aMarker->documentid . '_' . $aMarker->markerid] = $aMarker->totaltime;
            }
            if (!isset($students[$aMarker->userid]->user)){
                $aUser->firstname = $aMarker->firstname;
                $aUser->lastname = $aMarker->lastname;
                $aUser->username = $aMarker->username;
                $aUser->id = $aMarker->userid;
                $students[$aMarker->userid]->user = $aUser;
            }
        }
    }
    
    // check some extreme situations
    if (count($students) == 0){
        $textmessage = get_string('notrackingresults', 'coursetracking');
    }
    
    /*
    *
    * This is the HTML Table non graphic output generation
    *
    */
    if ($outputType == 'HTML'){

        print_heading(get_string('studenttracktitle', 'coursetracking'), 2);
        echo 'center';
        if (isset($textmessage)){
            echo "<span>{$textmessage}</span>";
            return;
        }
        echo '<div class="notavailablebox">'.get_string('notavailableinHTMLmode', 'coursetracking').'</div>';
        echo '</center>';

    } else {
    /*
    *
    * This is the GD generation alternative
    *
    */
        header("Content-type: image/png");
        header("Cache-control: no-cache");
        header("Expires: 0");
      
        // Searching for font files

        $fontFile = $CFG->dirroot . "/mod/coursetracking/fonts/arial.ttf";
        if (!file_exists($fontFile))
            $fontFile = $CFG->dirroot . "/fonts/{$ttffont}.ttf";
    
        // if fonts where not given by project, try with system fonts.

        if (!file_exists($fontFile))
            $fontFile = @$CFG->systemfonts . "/{$ttffont}.ttf";
    
        // output special situations messages 

        if (isset($textmessage)){
            $bounds = imagettfbbox(14, 0, $fontFile, $textmessage);
            $imageWidth = $bounds[2] - $bounds[0] + 2;
            $imageHeight = ($bounds[1] - $bounds[7]) * 1.4;
        } else {
            $daysToDisplay = ($calendarInfo->secsFromOrigin / 86400 < 21) ? 21 : $calendarInfo->secsFromOrigin / 86400 ; 
            $imageWidth = min($labelWidth + $columnWidth * $daysToDisplay, $maxImageWidth);
            $imageHeight = $columnLabelHeight  + $lineHeight * (count(array_keys($distinctMarkers)) + count($resources)) + 10 ;
        }
    
        $im = imagecreatetruecolor($imageWidth, $imageHeight);
        imageantialias($im, TRUE);
    
        // Assigning colors

        $colors['black'] = imagecolorallocate($im, 0, 0, 0);
        $colors['quiteblack'] = imagecolorallocate($im, 10, 10, 10);
        $colors['green'] = imagecolorallocate($im, 0, 240, 0);
        $colors['yellow'] = imagecolorallocate($im, 230, 230, 0);
        $colors['red'] = imagecolorallocate($im, 200, 0, 0);
        $colors['white'] = imagecolorallocate($im, 240, 240, 240);
        $colors['lightgray'] = imagecolorallocate($im, 200, 200, 200);
        $colors['gray'] = imagecolorallocate($im, 150, 150, 150);
        $colors['darkgray'] = imagecolorallocate($im, 100, 100, 100);
        $colors['blue'] = imagecolorallocate($im, 0, 0, 180);
        $colors['isme'] = imagecolorallocate($im, 160, 160, 0);
        
        imagefilledrectangle($im, 0, 0, $imageWidth, $imageHeight, $colors['white']);
        
        if (!isset($textmessage)){
            /// Creating graph
    
            // Creating title

            imagettftext($im, 18, 0, 10, 26, $colors['black'], $fontFile, $r);
        
            // draw background

            imagefilledrectangle($im, $labelWidth + 10, $columnLabelHeight, $imageWidth, $imageHeight, $colors['gray']);
            
            // TODO map to calendar defines if reusable ones

            $DAYS = array('D', 'L', 'M', 'M', 'J', 'V', 'S');
    
            // draw calendar grid

            for ($i = 1; $i <= $daysToDisplay ; $i++){
                $currentDay = ($calendarInfo->dayOfWeek + $i - 1 + $trackOffset) % 7;
                $dayColor = ($currentDay  == 0) ? $colors['quiteblack'] : $colors['lightgray'] ;
    
                // image is less large than required for that number of days
                if (($labelWidth + $i * $columnWidth + 3) > $maxImageWidth * 1.5 ) break;
    
                imageline($im, $labelWidth + $i * $columnWidth + 3, $columnLabelHeight, $labelWidth + $i * $columnWidth + 3, $imageHeight, $dayColor);
                imagettftext($im, 6, 0, $labelWidth + $i * $columnWidth + 1, $columnLabelHeight - 2, $colors['blue'], $fontFile, $DAYS[$currentDay]);
                $calendar = cal_from_jd(unixtojd($trackstartdate) + $i + $trackOffset, CAL_GREGORIAN);
                imagettftext($im, 6, 0, $labelWidth + $i * $columnWidth + 1, $columnLabelHeight - 15, $colors['blue'], $fontFile, $calendar['day']);
                if ($calendar['day'] == 1 || ($i == 1 && $calendar['day'] < 24 && !@$startMonthDisplayed)){
                    $startMonthDisplayed = true;
                    $im2 = imagecreatefrompng($CFG->dirroot . "/mod/coursetracking/gdgenerators/gradient.png");
                    imagecopy($im, $im2, $labelWidth + $i * $columnWidth - 1, $columnLabelHeight - 41,  0, 0, 99, 15);
                    imagettftext($im, 10, 0, $labelWidth + $i * $columnWidth + 1, $columnLabelHeight - 30, $colors['quiteblack'], $fontFile, get_string(strtolower($calendar['monthname']), 'coursetracking'));
                }
            }
    
            // draw labels for documents and markers

            $i = 1;
            foreach(array_keys($markers) as $aDocumentId){
                $y = $columnLabelHeight + ($i * $lineHeight);
                imagettftext($im, 8, 0, 5, $y, $colors['black'], $fontFile, $resources[$aDocumentId]);
                $i++;
                foreach(array_keys($markers[$aDocumentId]) as $aMarkerId){
                    $y = $columnLabelHeight + ($i * $lineHeight);
                    imagettftext($im, 7, 0, 15, $y, $colors['blue'], $fontFile, $aMarkerId);
                    imageline($im, 0, $y + 1, $imageWidth, $y + 1, $colors['lightgray']);
                    imageline($im, 0, $y  + 2, $imageWidth, $y + 2, $colors['darkgray']);
                	$i++;
                }
            }
    
        	foreach(array_keys($students) as $aStudentId){

        	    // is that me ?

        	    ($students[$aStudentId]->user->id == $USER->id) ? $color = "isme" : $color = 'black';
                // imagettftext($im, 10, 90, $x + 3, 118, $colors[$color], $fontFile, $students[$aStudentId]->user->firstname . " " . $students[$aStudentId]->user->lastname);
    
                // draw VENI curves

                if ($showVeni){
                    $j = 0;
                    foreach(array_keys($students[$aStudentId]->venis) as $aDate){
                        foreach($students[$aStudentId]->venis[$aDate] as $aMarkerTag){
                            $x = $labelWidth + ($aDate * $columnWidth / 86400 ) - $columnWidth / 2 - $trackOffset * $columnWidth;
                            if ($x < $labelWidth + 10) continue;
                        	$y = $columnLabelHeight + $lineHeight * ($distinctMarkers[$aMarkerTag] + 1) - $lineHeight / 2;
                        	drawBox($im, $x, $y, 'red', $colors, $j > 0);
                            $j++;
                        }
                    }
                }

                // draw VIDI curves

                if ($showVidi){
                    $j = 0;
                    foreach(array_keys($students[$aStudentId]->vidis) as $aDate){
                        foreach($students[$aStudentId]->vidis[$aDate] as $aMarkerTag){
                            $x = $labelWidth + ($aDate * $columnWidth / 86400) - $columnWidth / 2 - $trackOffset * $columnWidth;
                            if ($x < $labelWidth + 10) continue;
                        	$y = $columnLabelHeight + $lineHeight * ($distinctMarkers[$aMarkerTag] + 1) - $lineHeight / 2;
                        	drawBox($im, $x, $y, 'yellow', $colors, $j > 0);
                            imagefilledrectangle($im, $x + 5, $y, $x + 5 + $vidiTimeBarLength * ($students[$aStudentId]->vidiTime[$aMarkerTag] / $totalViewTimeMax), $y + 1, $colors['yellow']);
                            $j++;
                        }
                    }
                }

                // draw VICI curves

                if ($showVici){
                    $j = 0;
                    foreach(array_keys($students[$aStudentId]->vicis) as $aDate){
                        foreach($students[$aStudentId]->vicis[$aDate] as $aMarkerTag){
                            $x = $labelWidth + ($aDate * $columnWidth / 86400) - $columnWidth / 2 - $trackOffset * $columnWidth;
                            if ($x < $labelWidth + 10) continue;
                        	$y = $columnLabelHeight + $lineHeight * ($distinctMarkers[$aMarkerTag] + 1) - $lineHeight / 2;
                        	drawBox($im, $x, $y, 'green', $colors, $j > 0);
                            $j++;
                        }
                    }
                }
            }
        } else {
            imagettftext($im, 12, 0, 5, $imageHeight - 4, $colors['black'], $fontFile, $textmessage);
        }
    
      // delivering image
      imagepng($im);
      imagedestroy($im);
      
    }
?>