/*
* Reveal & feed-back project for Moodle
* 
* requirements : needs definition of the current_user global in environment.
* requirements : needs definition of the document_id global in environment.
* requirements : needs definition of the siteroot global in environment.
* 
* could default to in general applications :
* var document_id = document.location.href;
* document_id = document_id.replace(urlPrefixRegex, "");
*
* For moodle integration, current_user good choice is moodle current's user login 
* and document_id should be the resourceID in course
*
*/
// set debug to true to use code trace external window
var debug = false;

// an array to store triggers on use
var vidiTriggers = new Array();

// an array to store registered markers
var markers = new Array();

// an array to store marker status in the range [init,veni,vidi,vici]
var markerStates = new Array();

// an array to store programmed value of coursetracking Id
var trackerIds = new Array();

// an array to store programmed value of vidi timeouts
var markerDelais = new Array();

// an array to store current viewtime
var markerViewTime = new Array();

// sets the default delay if no explicit delay is mentionned (in mills)
var defaultDelay = 5;

// a regex used to cleanup some url schemes
var urlPrefixRegex = new RegExp("^http://" + document.location.hostname + "/");

// an array to store if vidi is explicitely marked by user (future development)
var markerModes = new Array();

// the main poller delay
var pollDelay = 750;

// the debugging window instance
var debugLogWindow;

// used by the debugging code to stop feeding the debugging window
function disableLog(){
  debug = 0;
  return 0;
}

// if debugging open the trace
if (debug){
	debugLogWindow = window.open('', 'logwin', 'width=300,height=800,scrollbars=1');
}

// makes a timeout object an registers it
function armVidiTrigger(markername, vidiDelay){
   if (debug && debugLogWindow) debugLogWindow.document.write("armed Vidi : " + vidiDelay + "<br>\n");
   vidiTriggers[markername] = setTimeout("markVidi('" + markername + "','" + vidiDelay + "')", vidiDelay * 1000);
}

// asks for loading the initial marker aspect according to the last recorded state
function loadMarker(markername){
   url = siteroot + "/mod/coursetracking/capture_api/status_ajax.php?r=" + escape(document_id) + "&trkid=" + trackerIds[markername] + "&id=" + escape(markername) + "&user=" + escape(current_user) + "&cmd=init";
   if (debug && debugLogWindow) debugLogWindow.document.write("firing : " + url + "<br>\n");
   launchHttpRequest(url);
}

// asks for marking a visible episode of the document nearby the marker
function markVeni(markername){
   url = siteroot + "/mod/coursetracking/capture_api/mark_ajax.php?r=" + escape(document_id) + "&trkid=" + trackerIds[markername] + "&id=" + escape(markername) + "&user=" + escape(current_user) + "&cmd=veni";
   if (debug && debugLogWindow) debugLogWindow.document.write("firing : " + url + "<br>\n");
   launchHttpRequest(url);
}

// asks for marking Vidi event. This is a vidiTrigger bounce
function markVidi(markername, delay){
   url = siteroot + "/mod/coursetracking/capture_api/mark_ajax.php?r=" + escape(document_id) + "&trkid=" + trackerIds[markername] + "&id=" + escape(markername) + "&user=" + escape(current_user) + "&cmd=vidi&time=" + delay; 
   launchHttpRequest(url);
}

// asks for marking Vici event. This is triggered explicitly by user on checkbox
function markVici(markername){
   if (debug && debugLogWindow) debugLogWindow.document.write("VICI : " + markername + "<br>\n");
   url = siteroot + "/mod/coursetracking/capture_api/mark_ajax.php?r=" + escape(document_id) + "&trkid=" + trackerIds[markername] + "&id=" + escape(markername) + "&user=" + escape(current_user) + "&cmd=vici"; 
   if (debug && debugLogWindow) debugLogWindow.document.write("firing : " + url + "<br>\n");
   markerObj = document.getElementById(markername);
   markerObj.classname = "vici_status";
   launchHttpRequest(url);
}

// arm the global poller for all markers. Must be called by the content at end of the page.
function armPoller(){
  setInterval(pollMarkers, pollDelay);
}

// the main poller function. Polls all registered markers to examine visibility
function pollMarkers(){
   var i = 0;
   if (debug && debugLogWindow) debugLogWindow.document.write("<hr><a href='Javascript:window.opener.disableLog();'>disable trace</a><br>\n");
   if (debug && debugLogWindow) debugLogWindow.document.write("markers : " + markers.length + "<br>\n");
   for(i = 0 ; i < markers.length ; i++){
	   markerObj = markers[i];
	   // IE browsers
	   if (window.ActiveXObject){
	      markername = markerObj.name;
	   }
	   // Gecko browsers
	   else{
	      markername = markerObj.id;
	   }
	   if (isVisible(markername) == 'true'){
	      if (markerStates[markername] == "init"){
	         markVeni(markername);
	      }
	      else if (markerStates[markername] == "veni"){
	      	(!markerDelais[markername]) ? delay = 10 : delay = markerDelais[markername] ;
	         if (!vidiTriggers[markername]) armVidiTrigger(markername, delay);
	      }
	      else{
	         markerViewTime[markername]++;
	      }
	   }
	   else{
	      if (markerStates[markername] == "init"){
	         if (vidiTriggers[markername]) clearTimeout(vidiTriggers[markername]);
	         delete vidiTriggers[markername];
	      }
	      else if (markerStates[markername] == "veni"){
	         if (vidiTriggers[markername]) clearTimeout(vidiTriggers[markername]);
	         delete vidiTriggers[markername];
	      }
	      else{
	         if(markerViewTime[markername] != 0){
	            markVidi(markername, Math.floor(markerViewTime[markername] * pollDelay / 1000));
	            markerViewTime[markername] = 0;
	         } 
	      } 
	   }
   }
}

// launches Http request for asynchronous handshaking
function launchHttpRequest(url){
   var xmlHttp = GetXmlHttpObject();
   if (xmlHttp == null) {
      alert ("Ajax Error");
      return;
   }
   xmlHttp.onreadystatechange = function(){
			if ((xmlHttp.readyState == 4) || (xmlHttp.readyState == 'complete')){
			   var response = xmlHttp.responseText;
			   var markername = '';
			   re1 = /markerid="(.*?)"/;
			   re2 = /phase="([a-zA-Z0-9_]*)"/;
			   capture1 = response.match(re1);
			   if (capture1 && capture1.length != 0){
			      markername = capture1[1];
			   }
			   else return ;
			   // alert(markername);
			   capture2 = response.match(re2)
			   phase = capture2[1];
	   		   var markerObj = document.getElementById(markername);
	   		   markerObj.innerHTML = xmlHttp.responseText;
	   		   if (markers[markername] == undefined){
   				  if (debug && debugLogWindow) debugLogWindow.document.write("register : " + markername + " " + phase + "<br>\n");
	   		      markers = markers.concat(markerObj);
	   		   }
	   		   markerStates[markername] = phase;
	   		   xmlHttp = null;
			}
		};
   xmlHttp.open("GET", url, true);
   xmlHttp.send(null);
}

// intanciates an XMLHttpRequest friendly with all browser types
function GetXmlHttpObject(){
   var objXmlHttp = null;
   // for Gecko browsers
   if (window.XMLHttpRequest){
      objXmlHttp = new XMLHttpRequest();
   }
   // for IE browsers
   else if (window.ActiveXObject){
      objXmlHttp = new ActiveXObject("Microsoft.XMLHTTP");
   }
   return objXmlHttp;
}

// get a marker boundary coordinates
function getBoundaries(objname){
    if (objname == '') return false;
    var obj = document.getElementById(objname);
    var output = new Object();
    output.width = parseInt(obj.style.width.replace('px', ''));
    output.height = parseInt(obj.style.height.replace('px', ''));
    output.absX = obj.offsetLeft;
    output.absY = obj.offsetTop;
    while(obj = obj.offsetParent){
        output.absX += obj.offsetLeft;
        output.absY += obj.offsetTop;
    }
    return output;
}

// tests visibility of the object identified by objectname
function isVisible(objname){
    var boundaries = getBoundaries(objname);
    if (!boundaries) return false;
    boundaries.absY -= f_clientYOffset();
    if (debug && debugLogWindow) debugLogWindow.document.write("client offset is : " + f_clientYOffset() + "<br>\n");
    if (debug && debugLogWindow) debugLogWindow.document.write("visibility of : " + boundaries.absX + ":" + boundaries.absY + ":w" + boundaries.width + ":h" + boundaries.height + "<br>\n");
    if (debug && debugLogWindow) debugLogWindow.document.write("in client " + f_scrollLeft() + ":" + f_scrollTop() + ":w" + f_clientWidth() + ":h" + f_clientHeight() + "<br>\n");
    if (parseInt(boundaries.absX) + parseInt(boundaries.width) < parseInt(f_scrollLeft()) ||
        parseInt(boundaries.absX) > f_scrollLeft() + f_clientWidth() ||
        parseInt(boundaries.absY) + parseInt(boundaries.height) < f_scrollTop() ||
        parseInt(boundaries.absY) > f_scrollTop() + f_clientHeight() ){
    	  if (debug && debugLogWindow) debugLogWindow.document.write(objname + " is HIDDEN<br>\n");
        return "false";
    }
    if (debug && debugLogWindow) debugLogWindow.document.write(objname + " is VISIBLE<br>\n");
    return "true";
} 

// the following functions are a softcomplex.com implementation for browser friendly graphic environment parameters
// get client offset related to the top location of the application window
function f_clientYOffset() {
	return f_filterResults (
		window.innerWidth ? window.outerWidth - window.innerWidth : 0,
		window.screenTop ? window.screenTop : 0,
		0
	);
}

function f_clientWidth() {
	return f_filterResults (
		window.innerWidth ? window.innerWidth : 0,
		document.documentElement ? document.documentElement.clientWidth : 0,
		document.body ? document.body.clientWidth : 0
	);
}

function f_clientHeight() {
	return f_filterResults (
		window.innerHeight ? window.innerHeight : 0,
		document.documentElement ? document.documentElement.clientHeight : 0,
		document.body ? document.body.clientHeight : 0
	);
}

function f_scrollLeft() {
	return f_filterResults (
		window.pageXOffset ? window.pageXOffset : 0,
		document.documentElement ? document.documentElement.scrollLeft : 0,
		document.body ? document.body.scrollLeft : 0
	);
}

function f_scrollTop() {
	return f_filterResults (
		window.pageYOffset ? window.pageYOffset : 0,
		document.documentElement ? document.documentElement.scrollTop : 0,
		document.body ? document.body.scrollTop : 0
	);
}

function f_filterResults(n_win, n_docel, n_body) {
	var n_result = n_win ? n_win : 0;
	if (n_docel && (!n_result || (n_result > n_docel)))
		n_result = n_docel;
	return n_body && (!n_result || (n_result > n_body)) ? n_body : n_result;
}

function showObject(theObj){
    if (debug && debugLogWindow) debugLogWindow.document.write("Object(" + theObj + "){\n");
    for(aMember in theObj){
        if (debug && debugLogWindow) debugLogWindow.document.write("\t[" + aMember + " => " + theObj[aMember] + "]\n");
    }
    if (debug && debugLogWindow) debugLogWindow.document.write("}\n");
}     
