<?php  // $Id: mod_form.php,v 1.1.2.2 2009/12/31 17:02:50 diml Exp $
/**
 * Form to define a new instance of lesson or edit an instance.
 * It is used from /course/modedit.php.
 *
 * @version $Id: mod_form.php,v 1.1.2.2 2009/12/31 17:02:50 diml Exp $
 * @license http://www.gnu.org/copyleft/gpl.html GNU Public License
 * @package lesson
 **/

require_once($CFG->dirroot.'/course/moodleform_mod.php');

class mod_coursetracking_mod_form extends moodleform_mod {

    function definition() {
        global $CFG, $COURSE;

        $mform    =& $this->_form;

//-------------------------------------------------------------------------------
        $mform->addElement('header', 'general', get_string('general', 'form'));

        $mform->addElement('text', 'name', get_string('name'), array('size'=>'64'));
        if (!empty($CFG->formatstringstriptags)) {
            $mform->setType('name', PARAM_TEXT);
        } else {
            $mform->setType('name', PARAM_CLEAN);
        }
        $mform->addRule('name', null, 'required', null, 'client');

        $mform->addElement('htmleditor', 'description', get_string('description'));
        $mform->setType('description', PARAM_RAW);
        $mform->addRule('description', null, 'required', null, 'client');
 
        $mform->addElement('format', 'format', get_string('format'));

        // Create a text box that can be enabled/disabled for lesson time limit
        $mform->addElement('checkbox', 'coursetrackingenable', '', get_string('coursetrackingenable', 'coursetracking'));
        $mform->addElement('checkbox', 'teachersseetrackers', '', get_string('teachersseetrackers', 'coursetracking'));

        $date = getdate(time());
        $dateoptions = array('startyear' => $date['year'] - 2, 
            'stopyear'  => $date['year'] + 4,
            'timezone'  => 99, 
            'applydst'  => true, 
            'optional'  => false
        );

        $mform->addElement('header', 'moduletimerange', get_string('moduletimerange', 'coursetracking'));

        $timestartgrp[] = &$mform->createElement('checkbox', 'timestartenable', '', get_string('enable', 'coursetracking'));
        $timestartgrp[] = &$mform->createElement('date_selector', 'timestart', '', get_string('start', 'coursetracking'), $dateoptions);
        $mform->addGroup($timestartgrp, 'timestartgrp', get_string('timestart', 'coursetracking'), array(' '), false);
        $mform->disabledIf('timestartgrp', 'timestartenable');

        $timeendgrp[] = &$mform->createElement('checkbox', 'timeendenable', '', get_string('enable', 'coursetracking'));
        $timeendgrp[] = &$mform->createElement('date_selector', 'timeend', '', get_string('end', 'coursetracking'), $dateoptions);
        $mform->addGroup($timeendgrp, 'timeendgrp', get_string('timeend', 'coursetracking'), array(' '), false);
        $mform->disabledIf('timeendgrp', 'timeendenable');

        $mform->addElement('header', 'capturetimerange', get_string('capturetimerange', 'coursetracking'));

        $capturestartgrp[] = &$mform->createElement('checkbox', 'capturestartenable', '', get_string('enable', 'coursetracking'));
        $capturestartgrp[] = &$mform->createElement('date_selector', 'capturestart', '', get_string('start', 'coursetracking'), $dateoptions);
        $mform->addGroup($capturestartgrp, 'capturestartgrp', get_string('capturestart', 'coursetracking'), array(' '), false);
        $mform->disabledIf('capturestartgrp', 'capturestartenable');

        $captureendgrp[] = &$mform->createElement('checkbox', 'captureendenable', '', get_string('enable', 'coursetracking'));
        $captureendgrp[] = &$mform->createElement('date_selector', 'captureend', '', get_string('end', 'coursetracking'), $dateoptions);
        $mform->addGroup($captureendgrp, 'captureendgrp', get_string('captureend', 'coursetracking'), array(' '), false);
        $mform->disabledIf('captureendgrp', 'captureendenable');

        $mform->addElement('header', 'tracktimerange', get_string('tracktimerange', 'coursetracking'));

        $trackstartgrp[] = &$mform->createElement('checkbox', 'trackstartenable', '', get_string('enable', 'coursetracking'));
        $trackstartgrp[] = &$mform->createElement('date_selector', 'trackstart', '', get_string('start', 'coursetracking'), $dateoptions);
        $mform->addGroup($trackstartgrp, 'trackstartgrp', get_string('trackstart', 'coursetracking'), array(' '), false);
        $mform->disabledIf('trackstartgrp', 'trackstartenable');

        $trackendgrp[] = &$mform->createElement('checkbox', 'trackendenable', '', get_string('enable', 'coursetracking'));
        $trackendgrp[] = &$mform->createElement('date_selector', 'trackend', '', get_string('end', 'coursetracking'), $dateoptions);
        $mform->addGroup($trackendgrp, 'trackendgrp', get_string('trackend', 'coursetracking'), array(' '), false);
        $mform->disabledIf('trackendgrp', 'trackendenable');

        $mform->addElement('header', 'othersettings', get_string('other', 'coursetracking'));

        $mform->addElement('text', 'linklabel', get_string('linklabel', 'coursetracking'), array('size'=>'64'));
//-------------------------------------------------------------------------------
        $features = new stdClass;
        $features->groups = false;
        $features->groupings = false;
        $features->groupmembersonly = false;
        $this->standard_coursemodule_elements($features);
//-------------------------------------------------------------------------------
// buttons
        $this->add_action_buttons();
    }

    /**
     * Enforce defaults here
     *
     * @param array $default_values Form defaults
     * @return void
     **/
    function data_preprocessing(&$default_values) {
    }

    /**
     * Enforce validation rules here
     *
     * @param object $data Post data to validate
     * @return array
     **/
    function validation($data, $files) {
        $errors = parent::validation($data, $files);

        if (!empty($data->timestartenabled) && !empty($data->trackstartenabled)){
            if ($data->trackstart < $data->timestart){
                $errors['trackstart'] = get_string('err_trackbeforestart');
            }
        }

        if (!empty($data->timeendenabled) && !empty($data->trackendenabled)){
            if ($data->trackend > $data->timeend){
                $errors['trackstart'] = get_string('err_trackafterend');
            }
        }

        if (!empty($data->timestartenabled) && !empty($data->timeendenabled)){
            if ($data->timestart > $data->timeend){
                $errors['timeend'] = get_string('err_endafterstart');
            }
        }

        if (!empty($data->trackstartenabled) && !empty($data->trackendenabled)){
            if ($data->trackstart > $data->trackend){
                $errors['trackstart'] = get_string('err_trackendafterstart');
            }
        }

        return $errors;
    }
}
?>
