<?PHP //$Id: restorelib.php,v 1.1.2.1 2009/12/28 19:12:17 diml Exp $

    /**
    * This php script contains all the stuff to backup/restore
    * coursetracking mods
    *
    * @package mod-coursetracking
    * @category mod
    * @author Valery Fremaux (valery@valeisti.fr)
    * 
    */

    //This is the "graphical" structure of the coursetracking mod:
    //
    //           coursetracking                                  
    //          (CL,pk->id)               
    //               |                       
    //               |
    //         coursetracking_event                            
    //      (UL,pk->id, fk->trackerid)
    //
    // Meaning: pk->primary key field of the table
    //          fk->foreign key to link with parent
    //          nt->nested field (recursive data)
    //          IL->instance level info
    //          CL->course level info
    //          UL->user level info
    //          files->table may have files
    //
    //-----------------------------------------------------------

    /**
    * restores a complete module
    * @param object $mod
    * @param object $restore
    * @uses $CFG
    */
    function coursetracking_restore_mods($mod, $restore) {
        global $CFG;

        $status = true;

        //Get record from backup_ids
        $data = backup_getid($restore->backup_unique_code, $mod->modtype, $mod->id);

        if ($data) {
            //Now get completed xmlized object
            $info = $data->info;
            //traverse_xmlize($info);                                                                     //Debug
            //print_object ($GLOBALS['traverse_array']);                                                  //Debug
            //$GLOBALS['traverse_array']="";                                                              //Debug

            //Now, build the COURSETRACKING record structure
            $coursetracking->course = $restore->course_id;
            $coursetracking->name = backup_todb($info['MOD']['#']['NAME']['0']['#']);
            $coursetracking->description = backup_todb($info['MOD']['#']['DESCRIPTION']['0']['#']);
            $coursetracking->coursetrackingenable = backup_todb($info['MOD']['#']['COURSETRACKINGENABLE']['0']['#']);
            $coursetracking->teachersseetrackers = backup_todb($info['MOD']['#']['TEACHERSSEETRACKERS']['0']['#']);
            $coursetracking->timestartenable = backup_todb($info['MOD']['#']['TIMESTARTENABLE']['0']['#']);
            $coursetracking->timeendenable = backup_todb($info['MOD']['#']['TIMEENDENABLE']['0']['#']);
            $coursetracking->timestart = backup_todb($info['MOD']['#']['TIMESTART']['0']['#']);
            $coursetracking->timeend = backup_todb($info['MOD']['#']['TIMEEND']['0']['#']);
            $coursetracking->capturestartenable = backup_todb($info['MOD']['#']['CAPTURESTARTENABLE']['0']['#']);
            $coursetracking->captureendenable = backup_todb($info['MOD']['#']['CAPTUREENDENABLE']['0']['#']);
            $coursetracking->capturestart = backup_todb($info['MOD']['#']['CAPTURESTART']['0']['#']);
            $coursetracking->captureend = backup_todb($info['MOD']['#']['CAPTUREEND']['0']['#']);
            $coursetracking->trackstartenable = backup_todb($info['MOD']['#']['TRACKSTARTENABLE']['0']['#']);
            $coursetracking->trackendenable = backup_todb($info['MOD']['#']['TRACKENDENABLE']['0']['#']);
            $coursetracking->trackstart = backup_todb($info['MOD']['#']['TRACKSTART']['0']['#']);
            $coursetracking->trackend = backup_todb($info['MOD']['#']['TRACKEND']['0']['#']);
            $coursetracking->linklabel = backup_todb($info['MOD']['#']['LINKLABEL']['0']['#']);
            $coursetracking->timecreated = backup_todb($info['MOD']['#']['TIMECREATED']['0']['#']);
            $coursetracking->timemodified = backup_todb($info['MOD']['#']['TIMEMODIFIED']['0']['#']);
                           
            //The structure is equal to the db, so insert the coursetracking
            $newid = insert_record ('coursetracking', $coursetracking);

            //Do some output     
            echo '<ul><li>' . get_string('modulename', 'coursetracking') . " \"" . $coursetracking->name . "\"<br>";
            backup_flush(300);

            if ($newid) {
                //We have the newid, update backup_ids
                backup_putid($restore->backup_unique_code, $mod->modtype, $mod->id, $newid);

                //Now check if want to restore user data and do it.
                if ($restore->mods['coursetracking']->userinfo) {
                    //Restore events
                    $status = $status && coursetracking_events_restore_mods($mod->id, $newid, $info, $restore);
                }                
            } else {
                $status = false;
            }

            //Finalize ul        
            echo '</ul>';

        } else {
            $status = false;
        }

        return $status;
    }

    /**
    * This function restores track events
    * @param int $old_coursetracking_id 
    * @param int $new_coursetracking_id
    * @param array $info
    * @param $restore
    * @uses $CFG 
    */
    function coursetracking_events_restore_mods($old_coursetracking_id, $new_coursetracking_id, $info, $restore) {
        global $CFG;

        $status = true;

        //Get the deck array
        if ($events = @$info['MOD']['#']['TRACKEVENTS']['0']['#']['TRACKEVENT']){

            //Iterate over events
            for($i = 0; $i < sizeof($events); $i++) {
                $event_info  = $events[$i];
                //traverse_xmlize($card_info);                                                               //Debug
                //print_object ($GLOBALS['traverse_array']);                                                  //Debug
                //$GLOBALS['traverse_array']="";                                                              //Debug
    
                //We'll need this later!!
                $oldid = backup_todb($event_info['#']['ID']['0']['#']);
    
                //Now, build the EVENT record structure
                $event->trackerid = $new_coursetracking_id;
                $event->userid = backup_todb($event_info['#']['USERID']['0']['#']);
                /*
                $module = backup_todb($event_info['#']['MODULE']['0']['#']);
                $recordid = backup_todb($event_info['#']['RECORDID']['0']['#']);
                */
                $event->documentid = backup_todb($event_info['#']['DOCUMENTID']['0']['#']);
                $event->markerid = backup_todb($event_info['#']['MARKERID']['0']['#']);
                $event->event = backup_todb($event_info['#']['EVENT']['0']['#']);
                $event->eventcount = backup_todb($event_info['#']['EVENTCOUNT']['0']['#']);
                $event->firstoccurred = backup_todb($event_info['#']['FIRSTOCCURRED']['0']['#']);
                $event->lastoccurred = backup_todb($event_info['#']['LASTOCCURRED']['0']['#']);
                $event->totaltime = backup_todb($event_info['#']['TOTALTIME']['0']['#']);
    
                //We have to recode the userid field
                $user = backup_getid($restore->backup_unique_code, 'user', $event->userid);
                if ($user) {
                    $event->userid = $user->new_id;
                }

                //We have to try recoding the documentid field as a translated coursemodule
                if ($event->documentid){
                    $cm = backup_getid($restore->backup_unique_code, 'course_modules', $event->documentid);
                    if ($cm){
                        $event->documentid = $cm->new_id;
                    } else {
                        echo "failed finding recoded cm";
                    }
                }
        
                //The structure is equal to the db, so insert the event
                $newid = insert_record ('coursetracking_event', $event);
    
                //Do some output
                if (($i+1) % 50 == 0) {
                    echo ".";
                    if (($i+1) % 1000 == 0) {
                        echo "<br />";
                    }
                    backup_flush(300);
                }
    
                if ($newid) {
                    //We have the newid, update backup_ids
                    backup_putid($restore->backup_unique_code, 'coursetracking_event', $oldid, $newid);
                } else {
                    $status = false;
                }
            }
        }
        return $status;
    }

    // TODO : recode properly some links.
?>